import { ServiceStore } from "./ServiceStore";
import { IService } from "./Service";
import { ActivityScope } from "./ActivityScope";
export interface StoreListener<StateT> {
    (newState: StateT, oldState: StateT): void;
}
export interface StoreSubscription {
    (): void;
}
export declare type ServiceOrStore = ServiceStore<any> | IService<any>;
export declare class SystemState {
    initialized: boolean;
    nextTranId: number;
    nextActivityId: number;
}
export interface StoreMiddleware {
    (activity: ActivityScope, next: StoreMiddlewareNext): any;
}
export interface StoreMiddlewareNext {
    (activity: ActivityScope): any;
}
export interface AppStoreConfig {
    activityAutoBeginTransaction?: boolean;
    updateAutoBeginTransaction?: boolean;
    allowConcurrencyErrors?: boolean;
}
export declare class AppStore<StateT extends object> {
    private listeners;
    private appState;
    private stores;
    private services;
    private activityListeners;
    private middleware;
    private middlewareNext;
    private _config;
    constructor();
    configure(config: AppStoreConfig): void;
    readonly config: AppStoreConfig;
    init(services: IService<any>[]): void;
    executeReactions(): void;
    private extractStore(serviceOrStore);
    getState(): StateT;
    subscribe(listener: StoreListener<StateT>): StoreSubscription;
    unsubscribe(listener: StoreListener<StateT>): void;
    commit(oldState: any, newState: any): StateT;
    emit(oldState: any, newState: any): void;
    private registerStore<StateT>(store);
    private findConflictingPath(path);
    registerListener(listener: ActivityListener): void;
    unregisterListener(listener: ActivityListener): void;
    registerMiddleware(middleware: (activity, next) => any): void;
    private emitActivityEvent(func, activity);
    _onActivityBegin(activity: ActivityScope): void;
    _runActivity(activity: ActivityScope, func: () => any): any;
    _onActivityError(activity: ActivityScope, err: any): void;
    _onActivitySuccess(activity: ActivityScope, res: any): void;
    _onActivitySyncComplete(activity: ActivityScope): void;
    _onActivityAsyncComplete(activity: ActivityScope): void;
    _onActivityZoneComplete(activity: ActivityScope): void;
}
export interface ActivityListener {
    onActivityBegin(activity: ActivityScope): any;
    onActivitySuccess(activity: ActivityScope, res: any): any;
    onActivityError(activity: ActivityScope, err: any): any;
    onActivitySyncComplete(activity: ActivityScope): any;
    onActivityAsyncComplete(activity: ActivityScope): any;
    onActivityZoneComplete(activity: ActivityScope): any;
}
